/*
 * Subverse
 * Copyright (C) 2007 Saulo Popov Zambiasi
 *
 * Subverse is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Subverse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: saulopz@gmail.com
 */

#ifndef WORLD_H_
#define WORLD_H_

#include "Graph.h"
#include "Globals.h"
#include "Avatar.h"
#include "Cyclope.h"
#include "Spider.h"
#include "Bat.h"
#include "Images.h"
#include "Timer.h"
#include "LoadWorld.h"
#include "Vision.h"

class Line {
public:
	vector<Element*> x;
};

class Type {
public:
	vector<Line*> y;
};

class World {
private:
    Graph *g;
    u_long idIndex;
	Image *background;
	vector<Type*> element;
	int posX;
	int posY;
	Element* avatar;
	Element* elementInfo;
	int height;
	int width;
	Timer *timer;
	int step;
	int turnGame;

	TTF_Font *font12;
	SDL_Color corBranca;

	void loadWorld(string);

public:
	Images *images;

	World(string);
	Timer *getTimer();
	void incTimer();
	Image *getImage();

	u_long getNextId();
	Element* get(int, int, int);
	Element* get(Element*, int, int);
	void set(int, int, int, Element*);
	bool move(Element*, int);
	int getWidth();
	int getHeight();
	int getScreenWidth();
	int getScreenHeight();
	int getX();
	int getY();
	int getTurnGame();
	void setMusic(bool);
	void setX(int);
	void setY(int);
	void setPosition(int, int);
	Vision* getVision(Element*, int);
	virtual void draw();
	virtual void run();
	virtual ~World();
};

#endif /*WORLD_H_*/
